package solutions.bitbadger.documents.groovy

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.AutoId
import solutions.bitbadger.documents.DocumentException
import solutions.bitbadger.documents.groovy.support.*

import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `AutoId` enum
 */
@DisplayName('JVM | Groovy | AutoId')
class AutoIdTest {

    @Test
    @DisplayName('Generates a UUID string')
    void generateUUID() {
        assertEquals('The UUID should have been a 32-character string', 32, AutoId.generateUUID().length())
    }

    @Test
    @DisplayName('Generates a random hex character string of an even length')
    void generateRandomStringEven() {
        def result = AutoId.generateRandomString 8
        assertEquals("There should have been 8 characters in $result", 8, result.length())
    }

    @Test
    @DisplayName('Generates a random hex character string of an odd length')
    void generateRandomStringOdd() {
        def result = AutoId.generateRandomString 11
        assertEquals("There should have been 11 characters in $result", 11, result.length())
    }

    @Test
    @DisplayName('Generates different random hex character strings')
    void generateRandomStringIsRandom() {
        def result1 = AutoId.generateRandomString 16
        def result2 = AutoId.generateRandomString 16
        assertNotEquals('There should have been 2 different strings generated', result1, result2)
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('needsAutoId fails for null document')
//    void needsAutoIdFailsForNullDocument() {
//        assertThrows(DocumentException) { AutoId.needsAutoId(AutoId.DISABLED, null, 'id') }
//    }
//
//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('needsAutoId fails for missing ID property')
//    void needsAutoIdFailsForMissingId() {
//        assertThrows(DocumentException) { AutoId.needsAutoId(AutoId.UUID, new IntIdClass(0), 'Id') }
//    }

    @Test
    @DisplayName('needsAutoId returns false if disabled')
    void needsAutoIdFalseIfDisabled() {
        assertFalse('Disabled Auto ID should always return false', AutoId.needsAutoId(AutoId.DISABLED, '', ''))
    }

    @Test
    @DisplayName('needsAutoId returns true for Number strategy and byte ID of 0')
    void needsAutoIdTrueForByteWithZero() {
        assertTrue('Number Auto ID with 0 should return true',
                AutoId.needsAutoId(AutoId.NUMBER, new ByteIdClass((byte) 0), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for Number strategy and byte ID of non-0')
    void needsAutoIdFalseForByteWithNonZero() {
        assertFalse('Number Auto ID with 77 should return false',
                AutoId.needsAutoId(AutoId.NUMBER, new ByteIdClass((byte) 77), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns true for Number strategy and short ID of 0')
    void needsAutoIdTrueForShortWithZero() {
        assertTrue('Number Auto ID with 0 should return true',
                AutoId.needsAutoId(AutoId.NUMBER, new ShortIdClass((short) 0), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for Number strategy and short ID of non-0')
    void needsAutoIdFalseForShortWithNonZero() {
        assertFalse('Number Auto ID with 31 should return false',
                AutoId.needsAutoId(AutoId.NUMBER, new ShortIdClass((short) 31), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns true for Number strategy and int ID of 0')
    void needsAutoIdTrueForIntWithZero() {
        assertTrue('Number Auto ID with 0 should return true',
                AutoId.needsAutoId(AutoId.NUMBER, new IntIdClass(0), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for Number strategy and int ID of non-0')
    void needsAutoIdFalseForIntWithNonZero() {
        assertFalse('Number Auto ID with 6 should return false',
                AutoId.needsAutoId(AutoId.NUMBER, new IntIdClass(6), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns true for Number strategy and long ID of 0')
    void needsAutoIdTrueForLongWithZero() {
        assertTrue('Number Auto ID with 0 should return true',
                AutoId.needsAutoId(AutoId.NUMBER, new LongIdClass(0L), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for Number strategy and long ID of non-0')
    void needsAutoIdFalseForLongWithNonZero() {
        assertFalse('Number Auto ID with 2 should return false',
                AutoId.needsAutoId(AutoId.NUMBER, new LongIdClass(2L), 'id'))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('needsAutoId fails for Number strategy and non-number ID')
//    void needsAutoIdFailsForNumberWithStringId() {
//        assertThrows(DocumentException) { AutoId.needsAutoId(AutoId.NUMBER, new StringIdClass(''), 'id') }
//    }

    @Test
    @DisplayName('needsAutoId returns true for UUID strategy and blank ID')
    void needsAutoIdTrueForUUIDWithBlank() {
        assertTrue('UUID Auto ID with blank should return true',
                AutoId.needsAutoId(AutoId.UUID, new StringIdClass(''), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for UUID strategy and non-blank ID')
    void needsAutoIdFalseForUUIDNotBlank() {
        assertFalse('UUID Auto ID with non-blank should return false',
                AutoId.needsAutoId(AutoId.UUID, new StringIdClass('howdy'), 'id'))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('needsAutoId fails for UUID strategy and non-string ID')
//    void needsAutoIdFailsForUUIDNonString() {
//        assertThrows(DocumentException) { AutoId.needsAutoId(AutoId.UUID, new IntIdClass(5), 'id') }
//    }

    @Test
    @DisplayName('needsAutoId returns true for Random String strategy and blank ID')
    void needsAutoIdTrueForRandomWithBlank() {
        assertTrue('Random String Auto ID with blank should return true',
                AutoId.needsAutoId(AutoId.RANDOM_STRING, new StringIdClass(''), 'id'))
    }

    @Test
    @DisplayName('needsAutoId returns false for Random String strategy and non-blank ID')
    void needsAutoIdFalseForRandomNotBlank() {
        assertFalse('Random String Auto ID with non-blank should return false',
                AutoId.needsAutoId(AutoId.RANDOM_STRING, new StringIdClass('full'), 'id'))
    }

//    TODO: resolve java.base open issue
//    @Test
//    @DisplayName('needsAutoId fails for Random String strategy and non-string ID')
//    void needsAutoIdFailsForRandomNonString() {
//        assertThrows(DocumentException.class) {
//            AutoId.needsAutoId(AutoId.RANDOM_STRING, new ShortIdClass((short) 55), 'id')
//        }
//    }
}
