package solutions.bitbadger.documents.groovy

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.Dialect
import solutions.bitbadger.documents.DocumentException

import static groovy.test.GroovyAssert.*

/**
 * Unit tests for the `Dialect` enum
 */
@DisplayName('JVM | Groovy | Dialect')
class DialectTest {

    @Test
    @DisplayName('deriveFromConnectionString derives PostgreSQL correctly')
    void derivesPostgres() {
        assertEquals('Dialect should have been PostgreSQL', Dialect.POSTGRESQL,
                Dialect.deriveFromConnectionString('jdbc:postgresql:db'))
    }

    @Test
    @DisplayName('deriveFromConnectionString derives SQLite correctly')
    void derivesSQLite() {
        assertEquals('Dialect should have been SQLite', Dialect.SQLITE,
                Dialect.deriveFromConnectionString('jdbc:sqlite:memory'))
    }

    @Test
    @DisplayName('deriveFromConnectionString fails when the connection string is unknown')
    void deriveFailsWhenUnknown() {
        try {
            Dialect.deriveFromConnectionString('SQL Server')
            fail('Dialect derivation should have failed')
        } catch (DocumentException ex) {
            assertNotNull 'The exception message should not have been null', ex.message
            assertTrue('The connection string should have been in the exception message',
                    ex.message.contains('[SQL Server]'))
        }
    }
}
