package solutions.bitbadger.documents.jvm.integration.postgresql

import solutions.bitbadger.documents.*
import solutions.bitbadger.documents.extensions.customNonQuery
import solutions.bitbadger.documents.extensions.customScalar
import solutions.bitbadger.documents.extensions.ensureTable
import solutions.bitbadger.documents.jvm.*
import solutions.bitbadger.documents.support.*

/**
 * A wrapper for a throwaway PostgreSQL database
 */
class PgDB : ThrowawayDatabase {

    private var dbName = ""

    init {
        DocumentConfig.serializer = JacksonDocumentSerializer()
        dbName = "throwaway_${AutoId.generateRandomString(8)}"
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use {
            it.customNonQuery("CREATE DATABASE $dbName")
        }
        Configuration.connectionString = connString(dbName)
    }

    override val conn = Configuration.dbConn()

    init {
        conn.ensureTable(TEST_TABLE)
    }

    override fun close() {
        conn.close()
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use {
            it.customNonQuery("DROP DATABASE $dbName")
        }
        Configuration.connectionString = null
    }

    override fun dbObjectExists(name: String) =
        conn.customScalar("SELECT EXISTS (SELECT 1 FROM pg_class WHERE relname = :name) AS it",
            listOf(Parameter(":name", ParameterType.STRING, name)), Boolean::class.java, Results::toExists)

    companion object {

        /**
         * Create a connection string for the given database
         *
         * @param database The database to which the library should connect
         * @return The connection string for the database
         */
        private fun connString(database: String) =
            "jdbc:postgresql://localhost/$database?user=postgres&password=postgres"
    }
}
